#!/usr/bin/env python3
from __future__ import annotations
import os.path
from io import BytesIO
import convert_stream as cs
import ocr_stream as ocr
import soup_files as sp


class Ocr(ocr.RecognizeImage):
    """
    Singleton para reconhecimento de texto em imagens
    """
    _instance = None  # armazena a instância única

    def __new__(
                cls, bin_tess: ocr.BinTesseract = ocr.BinTesseract(), *,
                lib_ocr: ocr.LibOcr = ocr.DEFAULT_LIB_OCR
            ):
        if cls._instance is None:
            # Cria a instância uma única vez
            cls._instance = super(Ocr, cls).__new__(cls)
        return cls._instance

    def __init__(
                self, bin_tess: ocr.BinTesseract = ocr.BinTesseract(), *,
                lib_ocr: ocr.LibOcr = ocr.DEFAULT_LIB_OCR,
            ):
        # Evita reexecutar __init__ em chamadas subsequentes
        if not hasattr(self, "_initialized"):
            super().__init__(bin_tess, lib_ocr=lib_ocr)
            self._initialized = True


def create_table_from_dict(data: dict[str, cs.ColumnBody]) -> cs.TextTable:
    _values: list[cs.ColumnBody] = []
    for _k in data.keys():
        _values.append(data[_k])
    return cs.TextTable(_values)


def recognize_images(
            images: list[ocr.ImageObject] | list[sp.File], *,
            recognize: Ocr = Ocr(),
            pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter(),
        ) -> cs.DocumentPdf:
    """
        Aplicar OCR em lista de imagens, e retornar um documento DocumentPdf() com as imagens embutidas.
    """
    pages_pdf: list[cs.PageDocumentPdf] = []
    max_num: int = len(images)
    print()
    pbar.start()
    for _num, im in enumerate(images):
        if isinstance(im, sp.File):
            im = cs.ImageObject(im)
        pbar.update(
            ((_num + 1) / max_num) * 100,
            f'[OCR IMAGEM]: {_num + 1}/{max_num} {im.metadata.name}',
        )
        tmp_doc = recognize.image_recognize(im).to_document()
        pages_pdf.extend(tmp_doc.to_pages())
        del tmp_doc
    pbar.stop()
    print()
    return cs.DocumentPdf.create_from_pages(pages_pdf)


def read_image(
            image: sp.File | cs.ImageObject | bytes | BytesIO, *,
            recognize: Ocr = Ocr(),
        ) -> dict[str, cs.ColumnBody]:
    """
        Extrair o texto de uma imagem e retornar o texto em no formato cs.ColumnBody().
    """
    if isinstance(image, sp.File):
        image: cs.ImageObject = cs.ImageObject(image)
    elif isinstance(image, cs.ImageObject):
        pass
    elif isinstance(image, bytes):
        image: cs.ImageObject = cs.ImageObject.create_from_bytes(image)
    elif isinstance(image, BytesIO):
        image: cs.ImageObject = cs.ImageObject(image)
    else:
        raise ValueError('Use: File|DocumentPdf|bytes|ByesIO')

    txt = recognize.image_to_string(image)
    try:
        _values = txt.split('\n')
    except Exception as e:
        print(f'Error: {e}')
        _values = ['nan']

    if image.metadata.file_path.is_empty:
        return cs.mod_types.table_types.create_dict_map_values(_values, file_type='.png')
    else:
        return cs.create_dict_map_from_file(
            sp.File(image.metadata.file_path), _values
        )


def read_images(
            images_files: list[cs.ImageObject] | list[sp.File], *,
            recognize: Ocr = Ocr(),
            pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter(),
        ) -> dict[str, cs.ColumnBody]:
    """
        Lista de imagens e retorna dict[str, ColumnBody]
    """
    list_table: list[dict[str, cs.ColumnBody]] = []
    max_num: int = len(images_files)
    print()
    pbar.start()
    for _num, img in enumerate(images_files):
        if isinstance(img, sp.File):
            img: cs.ImageObject = cs.ImageObject(img)
        pbar.update(
            ((_num + 1) / max_num) * 100,
            f'[OCR IMAGEM]: {_num + 1}/{max_num} {img.metadata.name}',
        )
        list_table.append(read_image(img, recognize=recognize))
    return cs.mod_types.table_types.concat_maps(list_table)


def read_directory_image(
            directory: sp.Directory, *,
            recognize: Ocr = Ocr(),
            pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter(),
        ) -> list[cs.TextTable]:
    """
        Ler as imagens de um diretório e retorna list[TextMap]
    """
    _files: list[sp.File] = sp.InputFiles(directory).get_files(file_type=sp.LibraryDocs.IMAGE)
    _data: list[cs.TextTable] = []
    max_num: int = len(_files)
    print()
    pbar.start()
    img: sp.File
    for _num, img in enumerate(_files):
        pbar.update(
            ((_num + 1) / max_num) * 100,
            f'[OCR]: {_num + 1}/{max_num} {img.basename()}',
        )
        _data.append(
            create_table_from_dict(read_image(img, recognize=recognize))
        )
    return _data


def read_document_pdf(
            document: cs.DocumentPdf,
            file_path: str,
            apply_ocr: bool = False,
            recognize: Ocr = Ocr(),
            pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter(),
            dpi: int = 200
        ) -> cs.TextTable:
    """
    Extrair os textos de páginas PDF e retornar um objeto dict[str, ColumnBody]
    """
    if not isinstance(document, cs.DocumentPdf):
        raise TypeError(f'file_pdf dev ser DocumentPdf() não {type(document)}')
    if apply_ocr:
        conv = cs.ConvertPdfToImages.create(document)
        conv.set_pbar(pbar)
        images: list[cs.ImageObject] = conv.to_images(dpi=dpi)
        document: cs.DocumentPdf = recognize_images(images, recognize=recognize, pbar=pbar)
    tb = document.to_dict()
    max_num = len(tb[cs.ColumnsTable.TEXT.value])
    tb[cs.ColumnsTable.FILE_PATH.value] = cs.ColumnBody(
        cs.ColumnsTable.FILE_PATH.value, [file_path] * max_num
    )
    tb[cs.ColumnsTable.DIR.value] = cs.ColumnBody(
        cs.ColumnsTable.DIR.value, [os.path.basename(file_path)] * max_num
    )
    tb[cs.ColumnsTable.FILE_NAME.value] = cs.ColumnBody(
        cs.ColumnsTable.FILE_NAME.value, [os.path.basename(file_path)] * max_num
    )
    return create_table_from_dict(tb)


def read_file_pdf(
            file_pdf: sp.File,
            apply_ocr: bool = False,
            recognize: Ocr = Ocr(),
            pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter(),
            dpi: int = 200
        ) -> cs.TextTable:
    """
    Extrair os textos de páginas PDF e retornar um objeto cs.TextTable()
    """
    if not isinstance(file_pdf, sp.File):
        raise TypeError(f'file_pdf dev ser File() não {type(file_pdf)}')
    tb = read_document_pdf(
        cs.DocumentPdf(file_pdf),
        file_pdf.absolute(),
        recognize=recognize,
        pbar=pbar,
        apply_ocr=apply_ocr,
        dpi=dpi,
    )
    return create_table_from_dict(tb)


def read_directory_pdf(
            directory: sp.Directory, *,
            apply_ocr: bool = False,
            recognize: Ocr = Ocr(),
            pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter(),
        ) -> list[cs.TextTable]:
    #
    files: list[sp.File] = sp.InputFiles(directory).get_files(file_type=sp.LibraryDocs.PDF)
    _text_maps: list[cs.TextTable] = []
    for f_pdf in files:
        _current_maps: dict[str, cs.ColumnBody] = read_file_pdf(
            f_pdf, apply_ocr=apply_ocr, recognize=recognize, pbar=pbar
        )
        _text_maps.append(create_table_from_dict(_current_maps))
    return _text_maps


__all__ = [
    'Ocr', 'read_image', 'read_images', 'read_directory_image',
    'read_document_pdf', 'read_file_pdf', 'read_directory_pdf',
    'create_table_from_dict'
]



#!/usr/bin/env python3

from __future__ import annotations
import pandas as pd
from soup_files import File, JsonConvert
import convert_stream as cs


class SearchableText(object):

    default_elements: dict[str, cs.ColumnBody] = cs.mod_types.table_types.create_void_dict_map()
    default_columns: cs.HeadValues = cs.HeadValues([cs.HeadCell(x) for x in list(default_elements.keys())])

    def __init__(self):
        self.elements: dict[str, cs.ColumnBody] = cs.create_void_dict_map()

    def __repr__(self):
        return f'SearchableText\nHead: {self.head}\nBody: {self.body}'

    def is_empty(self) -> bool:
        return len(self.elements[cs.HeadCell(cs.ColumnsTable.TEXT.value)]) == 0

    @property
    def head(self) -> cs.HeadValues:
        return cs.HeadValues([cs.HeadCell(x) for x in list(self.elements.keys())])

    @property
    def body(self) -> list[cs.ColumnBody]:
        return [cs.ColumnBody(cs.HeadCell(_k), self.elements[_k]) for _k in self.elements.keys()]

    @property
    def first(self) -> dict[str, str]:
        if self.is_empty():
            return {}
        cols: cs.HeadValues = self.head
        _first = {}
        for col in cols:
            _first[col] = self.elements[col][0]
        return _first

    @property
    def last(self) -> dict[str, str]:
        if self.is_empty():
            return {}
        cols = self.head
        _last = {}
        for col in cols:
            _last[col] = self.elements[col][-1]
        return _last

    @property
    def length(self) -> int:
        return len(self.elements[cs.HeadCell(cs.ColumnsTable.TEXT.value)])

    @property
    def files(self) -> cs.ColumnBody:
        return self.elements[cs.HeadCell(cs.ColumnsTable.FILE_PATH.value)]

    def get_item(self, idx: int) -> dict[str, str]:
        cols: cs.HeadValues = self.head
        try:
            _item = {}
            for col in cols:
                _item[col] = self.elements[col][idx]
            return _item
        except Exception as err:
            print(err)
            return {}

    def get_column(self, name: str) -> cs.ColumnBody:
        return self.elements[cs.HeadCell(name)]

    def add_line(self, line: dict[str, str]) -> None:
        cols_line: cs.HeadValues = cs.HeadValues([cs.HeadCell(x) for x in list(line.keys())])
        cols_origin: cs.HeadValues = self.head
        for col in cols_origin:
            if cols_line.contains(col, case=True, iqual=True):
                self.elements[col].append(line[col])

    def clear(self) -> None:
        for _k in self.elements.keys():
            self.elements[_k].clear()

    def to_string(self) -> str:
        """
            Retorna o texto da coluna TEXT em formato de string
        ou 'nas' em caso de erro nas = Not a String
        """
        try:
            return ' '.join(self.elements[cs.HeadCell(cs.ColumnsTable.TEXT.value)])
        except Exception as e:
            print(e)
            return 'nan'

    def to_data_frame(self) -> pd.DataFrame:
        return pd.DataFrame.from_dict(self.elements)

    def to_file_json(self, file: File):
        """Exporta os dados da busca para arquivo .JSON"""
        dt = JsonConvert.from_dict(self.elements).to_json_data()
        dt.to_file(file)

    def to_file_excel(self, file: File):
        """Exporta os dados da busca para arquivo .XLSX"""
        self.to_data_frame().to_excel(file.absolute(), index=False)

    @classmethod
    def create(cls, df: pd.DataFrame):
        cols: list[str] = df.columns.tolist()
        tb = {}
        for col in cols:
            tb[cs.HeadCell(col)] = cs.ColumnBody(col, df[col].astype('str').values.tolist())
        s = cls()
        s.elements = tb
        return s


class DocumentFinder(object):

    def __init__(self, text_maps: list[cs.TextTable]):
        self.text_maps: list[cs.TextTable] = text_maps
        # Coluna que contem o texto usado como filtro em cada linha da busca
        self._col_name_filter: str = 'FILTRO'
        # Coluna que contem o texto de filtro adicional
        self._col_include_filter: str = 'FILTRO ADICIONAL'

    def add_table(self, item: cs.TextTable) -> None:
        self.text_maps.append(item)

    def find(self, text: str, *, iqual: bool = False, case: bool = False) -> SearchableText:
        _data = []
        s = SearchableText()
        text_map: cs.TextTable
        for text_map in self.text_maps:
            arr = cs.ArrayString(text_map[cs.ColumnsTable.TEXT.value])
            _current_idx: int | None = arr.find_index(text, iqual=iqual, case=case)
            if _current_idx is not None:
                new_line: dict[str, str] = {
                    cs.ColumnsTable.KEY.value: text_map[cs.ColumnsTable.KEY.value][_current_idx],
                    cs.ColumnsTable.NUM_PAGE.value: text_map[cs.ColumnsTable.NUM_PAGE.value][_current_idx],
                    cs.ColumnsTable.NUM_LINE.value: text_map[cs.ColumnsTable.NUM_LINE.value][_current_idx],
                    cs.ColumnsTable.TEXT.value: text_map[cs.ColumnsTable.TEXT.value][_current_idx],
                    cs.ColumnsTable.FILETYPE.value: text_map[cs.ColumnsTable.FILETYPE.value][_current_idx],
                    cs.ColumnsTable.FILE_PATH.value: text_map[cs.ColumnsTable.FILE_PATH.value][_current_idx],
                    cs.ColumnsTable.DIR.value: text_map[cs.ColumnsTable.DIR.value][_current_idx],
                }
                s.add_line(new_line)
                break
        return s

    def find_all(self, text: str, iqual: bool = False, case: bool = False) -> SearchableText:
        _data = []
        s = SearchableText()
        for current_tb in self.text_maps:
            arr = cs.ArrayString(current_tb[cs.ColumnsTable.TEXT.value])
            _current_idx = arr.find_index(text, iqual=iqual, case=case)
            if _current_idx is not None:
                new_line = {
                    cs.ColumnsTable.KEY.value: current_tb[cs.ColumnsTable.KEY.value][_current_idx],
                    cs.ColumnsTable.NUM_PAGE.value: current_tb[cs.ColumnsTable.NUM_PAGE.value][_current_idx],
                    cs.ColumnsTable.NUM_LINE.value: current_tb[cs.ColumnsTable.NUM_LINE.value][_current_idx],
                    cs.ColumnsTable.TEXT.value: current_tb[cs.ColumnsTable.TEXT.value][_current_idx],
                    cs.ColumnsTable.FILETYPE.value: current_tb[cs.ColumnsTable.FILETYPE.value][_current_idx],
                    cs.ColumnsTable.FILE_PATH.value: current_tb[cs.ColumnsTable.FILE_PATH.value][_current_idx],
                    cs.ColumnsTable.DIR.value: current_tb[cs.ColumnsTable.DIR.value][_current_idx],
                }
                s.add_line(new_line)
        return s

    def find_from_data(
                self, df: pd.DataFrame, *,
                col_filter: str,
                col_new_filter: str = None
            ) -> SearchableText:
        """
            Filtrar vários textos em documentos/arquivos, incluindo dois tipos de filtros simultaneos.

        :df: DataFrame com os textos a serem usados como filtro apartir da coluna indicada
        :col_filter: string da coluna que contém os textos a serem filtrados
        :col_new_filter: string da coluna que contém um filtro adicional (opcional)

        @type df: pd.DataFrame
        @type col_filter: str
        @type col_new_filter: str
        @rtype: SearchableText
        """
        if col_new_filter is not None:
            df = df[[col_filter, col_new_filter]].dropna(subset=[col_filter]).astype('str')
        else:
            df = df[[col_filter]].dropna(subset=[col_filter]).astype('str')

        list_data: list[pd.DataFrame] = []
        row: pd.Series
        for i, row in df.iterrows():
            _searchable: pd.DataFrame = self.find_all(f'{row[col_filter]}').to_data_frame()
            if not _searchable.empty:
                if col_new_filter is not None:
                    new_searchable: pd.DataFrame = self.find_all(f'{row[col_new_filter]}').to_data_frame()
                    if not new_searchable.empty:
                        _searchable = pd.concat([_searchable, new_searchable])
                        _searchable[self._col_name_filter] = [row[col_filter]] * len(_searchable)
                        _searchable[self._col_include_filter] = [row[col_new_filter]] * len(_searchable)
                    else:
                        _searchable[self._col_name_filter] = [row[col_filter]] * len(_searchable)
                        _searchable[self._col_include_filter] = ['nan'] * len(_searchable)
                else:
                    _searchable[self._col_name_filter] = [row[col_filter]] * len(_searchable)
                    _searchable[self._col_include_filter] = ['nan'] * len(_searchable)
                list_data.append(_searchable)

        if len(list_data) > 0:
            return SearchableText.create(pd.concat(list_data))
        return SearchableText()



#!/usr/bin/env python3

__version__ = '1.2'
from .read import (
    read_image, read_images, read_document_pdf, create_table_from_dict,
    read_directory_pdf, read_directory_image, read_file_pdf,
)

from .find import SearchableText, DocumentFinder
from .document import (
    OrganizeInnerData, DocumentTextExtract, OrganizeInnerText,
    FilterData, Filter
)





#!/usr/bin/env python3
from __future__ import annotations
from typing import Union
from organize_stream.find import DocumentFinder
from organize_stream.read import (
    read_file_pdf, read_document_pdf, read_image, create_table_from_dict
)
import soup_files as sp
import convert_stream as cs
import pandas as pd
import shutil

FindItem = Union[str, list[str]]

#  
_bad_chars: list[str] = [
    '.', '!', ':', '?', '(', ')', '{', '}',
    '+', '#', '@', '<', '>', '/', '¢', ':',
    ',', '®', '“', "“", ';', '‘',
]

_remove_end_name: list[str] = ['-']
_remove_start_name: list[str] = ['-']


def fmt_str_file(
            filename: str, *,
            max_char: int = 80,
            upper_case: bool = True
        ) -> str:
    for c in _bad_chars:
        filename = filename.replace(c, '')
    for c in _remove_end_name:
        if filename[-1] == c:
            filename = filename[:-1]
    for c in _remove_start_name:
        while c in filename[0]:
            filename = filename[1:]
    while '--' in filename:
        filename = filename.replace('--', '-')
    if upper_case:
        filename = filename.upper()
    if len(filename) <= max_char:
        return filename
    return filename[0:max_char]


def move_list_files(
            mv_items: dict[str, list[sp.File]], *,
            replace: bool = False
        ) -> None:
    total_file = len(mv_items['src'])
    for idx, file in enumerate(mv_items['src']):
        output_path: sp.File = mv_items['dest'][idx]
        if not file.exists():
            print(f'[PULANDO]: {idx + 1} Arquivo não encontrado {file.absolute()}')
        if output_path.exists():
            if not replace:
                _count = 0
                origin_name = output_path.name_absolute()
                origin_ext = output_path.extension()
                while output_path.exists():
                    _count += 1
                    new_name = f'{origin_name}-{_count}{origin_ext}'
                    output_path = sp.File(new_name)
                del origin_name
                del origin_ext
        print(f'Movendo: {idx+1}/{total_file} {file.absolute()}')
        try:
            shutil.move(file.absolute(), output_path.absolute())
        except Exception as e:
            print(f'{e}')
        del output_path


# Sujeito notificador
class NotifyProvider(object):

    def __init__(self):
        self.observers: list = []

    def add_observer(self, observer) -> None:
        self.observers.append(observer)

    def send_notify(self, tb: cs.TextTable) -> None:
        for obs in self.observers:
            obs.receive_notify(tb)


# Sujeito Observador.
class Observer(object):

    def __init__(self):
        pass

    def receive_notify(self, notify: cs.TextTable) -> None:
        pass


class Filter(object):

    def __init__(
                self,
                find_txt: str,
                out_dir: sp.Directory, *,
                separator: str = ' ',
                case: bool = False,
                iqual: bool = False
            ):
        self.find_txt: str = find_txt
        self.out_dir: sp.Directory = out_dir
        self.case: bool = case
        self.iqual: bool = iqual
        self.separator: str = separator


class FilterData(Filter):

    def __init__(
                self,
                out_dir: sp.Directory, *,
                col_find: str,
                col_new_name: str,
                cols_in_name: list[str] = [],
                separator: str = ' ',
                case: bool = False,
                iqual: bool = False):
        super().__init__('nan', out_dir, separator=separator, case=case, iqual=iqual)
        self.col_find: str = col_find
        self.col_new_name: str = col_new_name
        self.cols_in_name: list[str] = cols_in_name


class DocumentTextExtract(NotifyProvider):
    """
        Extrair texto de arquivos, e converter em Excel/DataFrame
    """

    def __init__(self):
        super().__init__()
        self.tb_list: list[cs.TextTable] = []
        self.pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter()
        self.__count: int = 0

    @property
    def is_empty(self) -> bool:
        return len(self.tb_list) == 0

    @property
    def finder(self) -> DocumentFinder:
        return DocumentFinder(self.tb_list)

    def add_table(self, tb: cs.TextTable) -> None:
        self.tb_list.append(tb)
        self.__count += 1
        self.pbar.update_text(f'{__class__.__name__} Tabela adicionada: {self.__count}')
        self.send_notify(tb)

    def add_directory_pdf(
                self, 
                dir_pdf: sp.Directory, *, 
                apply_ocr: bool = False, 
                dpi: int = 200
            ):
        files = sp.InputFiles(dir_pdf).get_files(file_type=sp.LibraryDocs.PDF)
        total = len(files)
        for n, f in enumerate(files):
            self.pbar.update(
                ((n + 1) / total) * 100,
                f'{n+1}/{total} {f.basename()}',
            )
            tb = read_file_pdf(f, apply_ocr=apply_ocr, pbar=self.pbar, dpi=dpi)
            self.add_table(tb)

    def add_directory_image(self, dir_image: sp.Directory):
        files_images = sp.InputFiles(dir_image).get_files(file_type=sp.LibraryDocs.IMAGE)
        total = len(files_images)
        for idx, f in enumerate(files_images):
            self.pbar.update(
                ((idx + 1) / total) * 100,
                f'{idx+1}/{total} {f.basename()}',
            )
            tb: cs.TextTable = create_table_from_dict(read_image(f))
            self.add_table(tb)

    def add_file_pdf(self, file_pdf: sp.File, apply_ocr: bool = False):
        tb = read_file_pdf(file_pdf, pbar=self.pbar, apply_ocr=apply_ocr)
        self.add_table(tb)

    def add_file_image(self, file_image: sp.File):
        tb: cs.TextTable = create_table_from_dict(read_image(file_image))
        self.add_table(create_table_from_dict(tb))

    def add_image(self, image: cs.ImageObject):
        if not isinstance(image, cs.ImageObject):
            raise TypeError('Image must be an cs.ImageObject')
        _tb: cs.TextTable = create_table_from_dict(read_image(image))
        self.add_table(_tb)

    def add_document(
                self, document:
                cs.DocumentPdf, *,
                apply_ocr: bool = False,
                dpi: int = 200,
            ):
        self.add_table(
            read_document_pdf(
                document,
                document.metadata.file_path,
                pbar=self.pbar,
                apply_ocr=apply_ocr,
                dpi=dpi
            )
        )

    def to_data(self) -> pd.DataFrame:
        if len(self.tb_list) == 0:
            return cs.mod_types.table_types.create_void_df_map()
        
        _data: list[pd.DataFrame] = []
        for m in self.tb_list:
            _data.append(pd.DataFrame.from_dict(m))
        return pd.concat(_data).astype('str')
       
    def to_excel(self, file: sp.File) -> None:
        self.to_data().to_excel(file.absolute(), index=False)


class Organize(Observer):

    def __init__(self, filter_text: Filter):
        super().__init__()
        self._count: int = 0
        self.filter: Filter = filter_text
        self.extractor: DocumentTextExtract = DocumentTextExtract()
        self.extractor.add_observer(self)
        self.pbar: sp.ProgressBarAdapter = sp.ProgressBarAdapter()

    def _show_error(self, txt: str):
        print()
        self.pbar.update_text(f'{__class__.__name__} txt')

    def add_image(self, image: cs.ImageObject | sp.File):
        if isinstance(image, sp.File):
            self.extractor.add_file_image(image)
        elif isinstance(image, cs.ImageObject):
            self.extractor.add_image(image)
        else:
            self._show_error(f'Image must be an cs.ImageObject | sp.File')

    def add_images(self, images: list[cs.ImageObject] | list[sp.File]):
        total = len(images)
        for n, image in enumerate(images):
            if isinstance(image, sp.File):
                image = cs.ImageObject(image)
            self.pbar.update(
                ((n + 1) / total) * 100,
                f'[ADICIONANDO IMAGEM] {n+1}/{total} {image.metadata.name}'
            )
            self.add_image(image)

    def add_document(
                self,
                document: cs.DocumentPdf, *,
                apply_ocr: bool = True,
                dpi: int = 200
            ):
        self.extractor.add_document(document, apply_ocr=apply_ocr, dpi=dpi)

    def add_dir_pdf(
                self,
                path: sp.Directory, *,
                apply_ocr: bool = True,
                dpi: int = 200
            ):
        self.extractor.add_directory_pdf(path, apply_ocr=apply_ocr, dpi=dpi)

    def add_dir_image(self, path: sp.Directory):
        self.extractor.add_directory_image(path)

    def receive_notify(self, notify: cs.TextTable) -> None:
        pass


class OrganizeInnerText(Organize):
    """
    Mover/Renomear arquivos de acordo com padrões de texto presentes
    nos documentos/imagens.

    O padrão de texto a ser filtrado deve ser criado no objeto FilterText(). Se desejar
    filtrar mais de uma ocorrência nos documentos/imagens, separe as ocorrências com um '|'

    """

    def __init__(self, filter_text: Filter):
        super().__init__(filter_text)

    def receive_notify(self, notify: cs.TextTable) -> None:
        self._count += 1
        self.move_where_contains_text(notify)

    def move_where_contains_text(self, tb: cs.TextTable) -> None:
        """
        Mover/Renomear arquivos de acordo com padrões de texto presentes
        nos documentos/imagens.
        """
        data = pd.DataFrame.from_dict(tb)
        df = data[[cs.ColumnsTable.TEXT.value, cs.ColumnsTable.FILE_PATH.value]].astype('str')
        mv_items: dict[str, list[sp.File]] = {'src': [], 'dest': []}

        # Divide padrões múltiplos separados por "|"
        patterns = [p.strip() for p in self.filter.find_txt.split('|') if p.strip()]
        if not patterns:
            print(f'{__class__.__name__}: Nenhum padrão de busca válido informado.')
            return

        # Define padrão regex dependendo de "iqual"
        if self.filter.iqual:
            regex_pattern = '^(' + '|'.join(patterns) + ')$'
        else:
            regex_pattern = '(' + '|'.join(patterns) + ')'

        # Filtra linhas no DataFrame
        mask: pd.Series = df[cs.ColumnsTable.TEXT.value].str.contains(
            regex_pattern,
            case=self.filter.case,
            regex=True,
            na=False
        )
        matched_df = df[mask]
        total_matches = len(matched_df)

        if total_matches == 0:
            print(f'{__class__.__name__}: Nenhum arquivo encontrado com os padrões "{self.filter.find_txt}".')
            return
        print(f'{__class__.__name__}: {total_matches} ocorrência(s) encontrada(s).')

        # Para cada linha encontrada, gera nome limpo e adiciona à lista de movimentação
        for _, row in matched_df.iterrows():
            src_file = sp.File(row[cs.ColumnsTable.FILE_PATH.value])
            text_content = row[cs.ColumnsTable.TEXT.value]
            # Usa o texto da linha como base do novo nome
            new_file_name: str = fmt_str_file(text_content.strip())
            # Garante que o nome não fique vazio
            if not new_file_name:
                new_file_name = src_file.basename()
            dest_file = self.filter.out_dir.join_file(f"{new_file_name}{src_file.extension()}")
            mv_items['src'].append(src_file)
            mv_items['dest'].append(dest_file)
        move_list_files(mv_items)


class OrganizeInnerData(Organize):
    """
        Organizar os arquivos com base nos dados de uma tabela/DataFrame
    """

    def __init__(self, df: pd.DataFrame, filter_data: FilterData):
        """
        :param df: DataFrame com os dados de uma tabela/DataFrame, onde cada linha de determinada
        coluna será comparada com os textos presentes nos documentos.

        :param filter_data: FilterData com os nomes das colunas onde será buscado os textos a serem
        filtrados linha a linha.
        """
        super().__init__(filter_data)
        self.df: pd.DataFrame = df
        self.filter_data: FilterData = filter_data

    def receive_notify(self, notify: cs.TextTable) -> None:
        self._count += 1
        self.move_where_math_column(notify)

    def move_where_math_column(self, tb: cs.TextTable) -> None:
        """
            Mover arquivos conforme as ocorrências de texto encontradas na tabela/DataFrame df.
        o nome do novo arquivo será igual à ocorrência de texto da coluna 'col_find', podendo
        estender o nome com elementos de outras colunas, tais colunas podem ser informadas (opcionalmente)
        no parâmetro cols_in_name.
            Ex:
        Suponha que a tabela para renomear aquivos tenha a seguinte estrutura:

        A      B        C
        maça   Cidade 1 xxyyy
        banana Cidade 2 yyxxx
        mamão  Cidade 3 xyxyx

        Se passarmos os parâmetros col_find='A' e col_new_name='A' e o texto banana for
        encontrado no(s) documento, o novo nome do arquivo será banana. Caso incluir o parâmetro
        cols_in_name=['B'] o novo nome do arquivo será banana-Cidade 2 ou
        banana-Cidade 2-yyxxx (se incluir cols_in_name=['B', 'C']).

        """
        list_values_find = self.df[self.filter_data.col_find].astype('str').values.tolist()
        list_values_new_name = self.df[self.filter_data.col_new_name].astype('str').values.tolist()
        tb = pd.DataFrame.from_dict(tb)
        text_in_docs = tb[cs.ColumnsTable.TEXT.value].astype('str').values.tolist()
        text_file_names = tb[cs.ColumnsTable.FILE_PATH.value].astype('str').values.tolist()
        mv_items: dict[str, list[sp.File]] = {'src': [], 'dest': []}

        cols_include_names: list[list[str]] = []
        if len(self.filter_data.cols_in_name) > 0:
            for c in self.filter_data.cols_in_name:
                values_include: list[str] = self.df[c].astype('str').values.tolist()
                cols_include_names.append(values_include)

        for num, txt in enumerate(list_values_find):
            for num_idx, txt_doc in enumerate(text_in_docs):
                if txt in txt_doc:
                    src_file = sp.File(text_file_names[num_idx])
                    output_file = list_values_new_name[num]
                    if len(self.filter_data.cols_in_name) > 0:
                        new = ''
                        for element in cols_include_names:
                            new = f'{new}-{element[num]}'
                        output_file = f'{output_file}-{new}'
                    output_file = fmt_str_file(output_file)
                    output_path = self.filter_data.out_dir.join_file(f'{output_file}{src_file.extension()}')
                    mv_items['src'].append(src_file)
                    mv_items['dest'].append(output_path)
        move_list_files(mv_items)
